from django.urls import path, include

from .views import (
	LoginView, 
	SignupView,
	LogoutView,
	AuthenticatedView,
	)

app_name = 'iota_auth'
urlpatterns = [
    path('login/', LoginView.as_view(), name='login'),
    path('signup/', SignupView.as_view(), name='signup'),
    path('logout/', LogoutView.as_view(), name='logout'),
    path('', AuthenticatedView.as_view(), name='authenticated'),
]


from django import forms
from django.conf import settings

from .models import User

class LoginForm(forms.Form):

	password = None
	remember = forms.BooleanField(
		label='Remember Me',
		required=False
		)

class LoginSeedForm(LoginForm):

	password = forms.CharField(
		widget=forms.PasswordInput(),
		label='Seed', 
		max_length=1000
		)
	email = forms.EmailField(
		label='Email'
		)

class SignupForm(forms.Form):
	email = forms.EmailField(
		label='Email'
		)
	first_name = forms.CharField(
		label='First name', 
		max_length=150
		)
	last_name = forms.CharField(
		label='Last name', 
		max_length=150
		)
	seed = forms.CharField(
		widget=forms.PasswordInput(),
		label='Seed', 
		max_length=1000
		)


from django.db import models
from django.core.mail import send_mail
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.hashers import (
	check_password, is_password_usable, make_password,
)

from six import binary_type

"""
This module allows importing AbstractBaseUser even when django.contrib.auth is
not in INSTALLED_APPS.
"""
import unicodedata

from django.contrib.auth import password_validation
from django.contrib.auth.hashers import (
	check_password, is_password_usable, make_password,
)
from django.db import models
from django.utils.crypto import get_random_string, salted_hmac
from django.utils.translation import gettext_lazy as _

from django.contrib.auth.base_user import BaseUserManager
from django.contrib.auth.models import AbstractUser, PermissionsMixin


class IotaBaseUserManager(BaseUserManager):

	def make_random_password(self, length=81,
							 allowed_chars='ABCDEFGHJKLMNPQRSTUVWXYZ9'):
		"""
		Generate a random password with the given length and given
		allowed_chars. The default value of allowed_chars does not have "I" or
		"O" or letters and digits that look similar -- just to avoid confusion.
		"""
		return get_random_string(length, allowed_chars)

class IotaUserManager(IotaBaseUserManager):
	use_in_migrations = True

	def _create_user(self, email, password, **extra_fields):
		"""Create and save a User with the given email and password."""
		if not email:
			raise ValueError('The given email must be set')
		email = self.normalize_email(email)
		user = self.model(email=email, **extra_fields)
		user.set_password(password)
		user.save(using=self._db)
		return user

	def create_user(self, email, password=None, **extra_fields):
		"""Create and save a regular User with the given email and password."""
		extra_fields.setdefault('is_staff', False)
		extra_fields.setdefault('is_superuser', False)
		return self._create_user(email, password, **extra_fields)

	def create_superuser(self, email, password, **extra_fields):
		"""Create and save a SuperUser with the given email and password."""
		extra_fields.setdefault('is_staff', True)
		extra_fields.setdefault('is_superuser', True)

		if extra_fields.get('is_staff') is not True:
			raise ValueError('Superuser must have is_staff=True.')
		if extra_fields.get('is_superuser') is not True:
			raise ValueError('Superuser must have is_superuser=True.')

		return self._create_user(email, password, **extra_fields)

class User(AbstractUser):

	username = None
	password = models.CharField(_('password'), max_length=128, unique=True)
	address = models.CharField(_('address'), max_length=128, blank=True)
	email = models.EmailField(_('email address'), blank=True, unique=True)
	first_name = models.CharField(_('first name'), max_length=30, blank=True)
	last_name = models.CharField(_('last name'), max_length=30, blank=True)
	date_joined = models.DateTimeField(_('date joined'), auto_now_add=True)
	is_staff = models.BooleanField(_('staff status'),default=False,help_text=_('Designates whether the user can log into this admin site.'),)
	is_active = models.BooleanField(_('active'),default=True,help_text=_('Designates whether this user should be treated as active. Unselect this instead of deleting accounts.'),)
	avatar = models.ImageField(upload_to='avatars/', null=True, blank=True)

	objects = IotaUserManager()

	USERNAME_FIELD = 'email'
	REQUIRED_FIELDS = ['password']

	class Meta:
		verbose_name = _('user')
		verbose_name_plural = _('users')

	def get_email(self):
		return self.email

	def get_full_name(self):
		full_name = '%s %s' % (self.first_name, self.last_name)
		return full_name.strip()

	def get_short_name(self):
		return self.first_name

	def email_user(self, subject, message, from_email=None, **kwargs):
		send_mail(subject, message, from_email, [self.email], **kwargs)

	# this methods are require to login super user from admin panel
	def has_perm(self, perm, obj=None):
		return self.is_staff
 
	# this methods are require to login super user from admin panel
	def has_module_perms(self, app_label):
		return self.is_staff

	@property
	def is_anonymous(self):
		return False

	@property
	def is_authenticated(self):
		return True


from django.core.exceptions import ValidationError
from django.utils.translation import gettext as _

class IotaSeedValidator:

	def __init__(self, exact_length=81):
		self.exact_length = exact_length

	def validate(self, password, user=None):
		# Check if length is less than 81 Characters
		if len(password) < self.exact_length:
			raise ValidationError(
				_("This seed must contain at least %(exact_length)d characters. The entered seed is to short"),
				code='password_too_short',
				params={'exact_length': self.exact_length},
				)
		# Check if length is greater than 81 Characters
		if len(password) > self.exact_length:
			raise ValidationError(
				_("This seed must contain exactly %(exact_length)d characters. The entered seed is to long"),
				code='password_too_long',
				params={'exact_length': self.exact_length},
				)
		# Check if any of the characters are lower case
		for char in password:
			if char.islower():
				raise ValidationError(
					_("This seed must contain all upper case Latin characters."),
					code='password_contains_lower_case',
					)
		# Check if any of the characters are digits 0-8
		banned_digits = ['0','1','2','3','4','5','6','7','8']
		for digit in banned_digits:
			if digit in password:
				raise ValidationError(
					_("This seed can only contain the letter 9, please remove all letters 0 to 8."),
					code='password_contains_forbidden_digits',
					)
		# Check the password contains a 9
		approved_digit = ['9']
		if approved_digit[0] not in password:
			raise ValidationError(
				_("This seed must contain the letter 9 at least once."),
				code='password_does_not_contains_nine',
				)		

	def get_help_text(self):
		if self.code == 'password_too_short':
			return _("Your password must contain exactly %(exact_length)d characters." % {'exact_length': self.exact_length})
		elif self.code == 'password_too_long':
			return _("This seed must contain exactly %(exact_length)d characters. The entered seed is to long" % {'exact_lenght':self.exact_length})
		elif self.code == 'password_contains_lower_case':
			return _("This seed must contain all upper case Latin characters.")
		else:
			return _("This seed must contain the letter 9 at least once.")


from setuptools import setup
setup(
  name = 'iota_auth',
  packages = ['iota_auth'], # this must be the same as the name above
  version = '0.1.1',
  description = 'A custom authentication backend for Django Users to connect to the IOTA Tangle.',
  author = 'David Guinane',
  author_email = 'david.j.guinane@gmail.com',
  url = 'https://github.com/davidjguinane', # use the URL to the github repo
  download_url = 'https://github.com/davidjguinane/django-iota-auth/archive/0.1.tar.gz', # I'll explain this in a second
  keywords = ['iota', 'django', 'authentication'], # arbitrary keywords
  classifiers = [
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Framework :: Django :: 2.0',
    'Natural Language :: English',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)', # example license
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3.6',
  ],
  install_requires=[
  'django-encrypted-cookie-session >= 3.2.0',
  'pyota',
  'six >=1.11.8',
  'filters >= 1.3.2',
  'request >= 2.18.4',
  'pysha3 >= 1.0.2',
  ]
)